/*
 
Copyright (C) 2006 NTT DATA Corporation
 
This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, version 2.
 
This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.
 
*/

package com.clustercontrol.collectiverun.composite;

import java.rmi.AccessException;
import java.util.HashMap;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.clustercontrol.bean.Property;
import com.clustercontrol.bean.SizeConstant;
import com.clustercontrol.collectiverun.action.GetParameterProperty;
import com.clustercontrol.collectiverun.action.GetType;
import com.clustercontrol.composite.PropertySheet;
import com.clustercontrol.util.Messages;

/**
 * ¹Է̾ܺ٥ݥå
 * 
 * @version 2.1.0
 * @since 1.0.0
 *  
 */
public class ParameterViewComposite extends Composite {

    /** ץѥƥ */
    protected PropertySheet propertySheet = null;

    /** ܥܥå */
    protected Combo type = null;
    
    /** ̥ϥåޥå */
    protected HashMap<String, String> typeMap = null;
    
    /** ʸ */
    protected List<String> typeList = null;
    
    private boolean openAccessInformation = false;
    
    /**
     * 󥹥ȥ饯
     * 
     * @param parent
     * @param style
     * @since 1.0.0
     */
    public ParameterViewComposite(Composite parent, int style) {
        super(parent, style);
        initialize();
    }

    /**
     * 
     * 
     * @since 2.1.0
     */
    private void initialize() {
        GridLayout layout = new GridLayout(1, true);
        this.setLayout(layout);
        layout.marginHeight = 0;
        layout.marginWidth = 0;

        Composite composite = new Composite(this, SWT.NONE);
        RowLayout compositeLayout = new RowLayout(SWT.HORIZONTAL);
        composite.setLayout(compositeLayout);
        compositeLayout.marginHeight = 0;
        compositeLayout.marginWidth = 0;

        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.verticalAlignment = GridData.FILL;
        composite.setLayoutData(gridData);

        Label management = new Label(composite, SWT.LEFT);
        management.setText(Messages.getString("type") + " : ");
        RowData rowData = new RowData();
        management.setLayoutData(rowData);

        this.type = new Combo(composite, SWT.READ_ONLY);
        rowData = new RowData(160, SizeConstant.SIZE_COMBO_HEIGHT);
        this.type.setLayoutData(rowData);
        
        //̾
        GetType getType = new GetType();
        try {
            typeMap = getType.getTypeHashMap();
        } catch (AccessException e) {
        	if(!openAccessInformation){
        		openAccessInformation = true;
        		// ʤξ硢顼ɽ
        		MessageDialog.openInformation(null, Messages.getString("message"),
	                    Messages.getString("message.accesscontrol.16"));
        	}
        }
        try {
        	typeList = getType.getTypeStringList();
        } catch (AccessException e) {
        	if(!openAccessInformation){
        		openAccessInformation = true;
        		// ʤξ硢顼ɽ
        		MessageDialog.openInformation(null, Messages.getString("message"),
	                    Messages.getString("message.accesscontrol.16"));
        	}
        }
        
        for(int i = 0; typeList != null && i < typeList.size(); i++){
        	this.type.add(typeList.get(i));
        }

        //Լ
        this.type.addSelectionListener(new SelectionListener() {
            /*
             * (non-Javadoc)
             * 
             * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
             */
            public void widgetSelected(SelectionEvent e) {
                update();
            }

            /*
             * (non-Javadoc)
             * 
             * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
             */
            public void widgetDefaultSelected(SelectionEvent e) {

            }
        });
        this.type.select(0);

        TableTree table = new TableTree(this, SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.FULL_SELECTION | SWT.MULTI);
        gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.verticalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.grabExcessVerticalSpace = true;
        gridData.horizontalSpan = 1;
        table.setLayoutData(gridData);

        this.propertySheet = new PropertySheet(table);

        //ץѥƥɽ
        update();
    }

    /**
     * ӥ塼
     * 
     * @since 2.1.0
     */
    public void update() {
        String typeId = getTypeId();

        Property data = null;
        try {
        	data = new GetParameterProperty().getProperty(typeId);
        } catch (AccessException e) {
        	if(!openAccessInformation){
        		openAccessInformation = true;
        		// ʤξ硢顼ɽ
        		MessageDialog.openInformation(null, Messages.getString("message"),
	                    Messages.getString("message.accesscontrol.16"));
        	}
        }
        
        if(data == null){
        	data = new Property(null, null, "");
        }
        
        this.propertySheet.setInput(data);

        this.propertySheet.expandAll();
    }

    /**
     * ӥ塼
     * 
     * @since 2.1.0
     */
    public void update(String sessionId) {
        GetParameterProperty parameter = new GetParameterProperty();
        if (sessionId != null && sessionId.length() > 0) {
            String typeId = null;
            try {
            	typeId = parameter.getTypeId(sessionId);
            } catch (AccessException e) {
            	if(!openAccessInformation){
            		openAccessInformation = true;
            		// ʤξ硢顼ɽ
            		MessageDialog.openInformation(null, Messages.getString("message"),
    	                    Messages.getString("message.accesscontrol.16"));
            	}
            }
            
            if (typeId != null) {
                Property data = null;
                try {
                	data = parameter.getPropertyBySessionId(sessionId);
                } catch (AccessException e) {
                	if(!openAccessInformation){
                		openAccessInformation = true;
                		// ʤξ硢顼ɽ
                		MessageDialog.openInformation(null, Messages.getString("message"),
        	                    Messages.getString("message.accesscontrol.16"));
                	}
                }

                if(data == null){
                	data = new Property(null, null, "");
                }
                
                this.propertySheet.setInput(data);

                this.propertySheet.expandAll();

                type.setText(getTypeString(typeId));
            }
        }
    }

    /**
     * ץѥƥ
     * 
     * @return ǡǥ
     */
    public Property getInputData() {
        return (Property) this.propertySheet.getInput();
    }

    /**
     * ץѥƥ
     * 
     * @param data
     */
    public void setInputData(Property data) {
    	this.propertySheet.setInput(data);

        this.propertySheet.expandAll();
    }

    /**
     * ID
     * 
     * @return
     */
    public String getTypeId() {
    	if(typeMap != null){
    		return typeMap.get(this.type.getText());
    	}
    	else{
    		return null;
    	}
    }
    
    /**
     * ʸ
     * 
     * @return
     */
    public String getTypeString(String typeId) {
    	if(typeMap != null){
    		return typeMap.get(typeId);
    	}
    	else{
    		return null;
    	}
    }
    
    /**
     * ΥݥåȤѤơ֥
     * 
     * @return ơ֥
     */
	public TableTree getTableTree() {
		return this.propertySheet.getTableTree();
	}
}