/*

 Copyright (C) 2006 NTT DATA Corporation

 This program is free software; you can redistribute it and/or
 Modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation, version 2.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 */

package com.clustercontrol.performanceMGR.monitor.ejb.session;

import java.rmi.RemoteException;
import java.security.Principal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Locale;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.quartz.SchedulerException;

import com.clustercontrol.monitor.run.bean.MonitorInfo;
import com.clustercontrol.performanceMGR.monitor.factory.AddMonitorPerformance;
import com.clustercontrol.performanceMGR.monitor.factory.DeleteMonitorPerformance;
import com.clustercontrol.performanceMGR.monitor.factory.ModifyMonitorPerformance;
import com.clustercontrol.performanceMGR.monitor.factory.SelectMonitorPerformance;

/**
 * @ejb.bean name="MonitorPerfController"
 *           jndi-name="MonitorPerfController"
 *           type="Stateless"
 *           transaction-type="Container"
 * 
 * @jboss.depends name="jboss.j2ee:service=EJB,jndiName=AccessController"
 * @jboss.depends name="jboss.j2ee:service=EJB,jndiName=RepositoryController"
 * @jboss.depends name="jboss.j2ee:service=EJB,jndiName=MonitorController"
 * @jboss.depends name="jboss.j2ee:service=EJB,jndiName=NotifyController"
 * @jboss.depends name="jboss.j2ee:service=EJB,jndiName=MonitorInfo"
 * 
 * @ejb.permission
 *     unchecked="true"
 *     method-intf="Home"
 *     
 * @ejb.permission
 *     unchecked="true"
 *     method-intf="LocalHome"
 *     
 * @ejb.permission
 *     unchecked="true"
 *     method-intf="Local"   
 *     
 */
public abstract class MonitorPerfControllerBean implements SessionBean {
	
	protected static Log m_log = LogFactory.getLog( MonitorPerfControllerBean.class );
	
	@SuppressWarnings("unused")
	private SessionContext m_context;
	
	public void setSessionContext(SessionContext ctx) throws EJBException, RemoteException {
		m_context = ctx;
	}
	
	/**
	 * ʹƻϿޤ
	 * 
	 * @ejb.interface-method
	 * 
	 * @ejb.permission
     *     role-name="PerformanceWrite"
     *     method-intf="Remote"
	 * 
	 * @param info ξ
	 * @return Ͽ硢true
	 * @throws CreateException
	 * @throws FinderException
	 * @throws SchedulerException
	 * @throws ParseException
	 * @throws NamingException
	 * @throws NotSupportedException
	 * @throws HeuristicMixedException
	 * @throws HeuristicRollbackException
	 * @throws RollbackException
	 * @throws InvalidTransactionException
	 * @throws IllegalStateException
	 * @throws SystemException
	 */
	public boolean addPerformance(MonitorInfo info) throws CreateException, FinderException, SchedulerException, ParseException, NamingException, NotSupportedException, HeuristicMixedException, HeuristicRollbackException, RollbackException, InvalidTransactionException, IllegalStateException, SystemException {
		Principal pri = m_context.getCallerPrincipal();

		// ʹƻϿ
		AddMonitorPerformance addPerf = new AddMonitorPerformance();
		return  addPerf.add(info, pri.getName());
	}
	
	/**
	 * ǽʹƻ򹹿ޤ
	 * 
	 * @ejb.interface-method
	 * 
	 * @ejb.permission
     *     role-name="PerformanceWrite"
     *     method-intf="Remote"
	 * 
	 * @param info ξ
	 * @return ѹ硢true
	 * @throws CreateException
	 * @throws FinderException
	 * @throws RemoveException
	 * @throws SchedulerException
	 * @throws ParseException
	 * @throws NamingException
	 * @throws NotSupportedException
	 * @throws HeuristicMixedException
	 * @throws HeuristicRollbackException
	 * @throws RollbackException
	 * @throws InvalidTransactionException
	 * @throws IllegalStateException
	 * @throws SystemException
	 */
	public boolean modifyPerformance(MonitorInfo info) throws CreateException, FinderException, RemoveException, SchedulerException, ParseException, NamingException, NotSupportedException, HeuristicMixedException, HeuristicRollbackException, RollbackException, InvalidTransactionException, IllegalStateException, SystemException {
		Principal pri = m_context.getCallerPrincipal();
		
		// ǽʹƻ򹹿
		ModifyMonitorPerformance modifyPerf = new ModifyMonitorPerformance();
		return  modifyPerf.modify(info, pri.getName());
	}

	/**
	 * ǽʹƻޤ
	 * 
	 * @ejb.interface-method
	 * 
	 * @ejb.permission
     *     role-name="PerformanceWrite"
     *     method-intf="Remote"
	 * 
	 * @param monitorId ƻID
	 * @return 硢true
	 * @throws FinderException
	 * @throws RemoveException
	 * @throws SchedulerException
	 * @throws NamingException
	 * @throws NotSupportedException
	 * @throws HeuristicMixedException
	 * @throws HeuristicRollbackException
	 * @throws RollbackException
	 * @throws InvalidTransactionException
	 * @throws IllegalStateException
	 * @throws SystemException
	 */
	public boolean deletePerformance(String monitorId) throws FinderException, RemoveException, SchedulerException, NamingException, NotSupportedException, HeuristicMixedException, HeuristicRollbackException, RollbackException, InvalidTransactionException, IllegalStateException, SystemException{

		// ǽʹƻ
		DeleteMonitorPerformance delPerf = new DeleteMonitorPerformance();
		return  delPerf.delete("PERF", monitorId);
	}
	
	/**
	 * ǽʹƻ֤ޤ
	 * 
	 * @ejb.interface-method
	 * 
	 * @ejb.permission
     *     role-name="PerformanceRead"
     *     method-intf="Remote"
	 * 
	 * @jboss.method-attributes
	 *     read-only="true"
	 * 
	 * @param monitorId ִƻID
	 * @return ǽʹƻ
	 * @throws CreateException
	 * @throws FinderException
	 * @throws SchedulerException
	 * @throws NamingException
	 */
	public MonitorInfo getPerformance(String monitorId) throws CreateException, FinderException, SchedulerException, NamingException {

		// ǽʹƻ
		SelectMonitorPerformance selectPerf = new SelectMonitorPerformance();
		MonitorInfo info = selectPerf.getMonitor("PERF", monitorId);

		return info;
	}

	/**
	 * ǽʹƻꥹȤ֤ޤ
	 * 
	 * @ejb.interface-method
	 * 
	 * @ejb.permission
     *     role-name="PerformanceRead"
     *     method-intf="Remote"
     *     
     * @jboss.method-attributes
	 *     read-only="true"
	 * 
	 * @return Object2
	 * @throws CreateException
	 * @throws FinderException
	 * @throws SchedulerException
	 * @throws NamingException
	 */
	public ArrayList getPerformanceList() throws CreateException, FinderException, SchedulerException, NamingException {

		// ǽʹƻ
		SelectMonitorPerformance selectPerf = new SelectMonitorPerformance();
		ArrayList list = selectPerf.getMonitorList("PERF");

		return list;
	}
	
	/**
	 * ǽʹƻΥơ֥֤ޤ
	 * 
	 * @ejb.interface-method
	 * 
	 * @ejb.permission
     *     unchecked="true"
     *     method-intf="Remote"
	 * 
	 * @param local 
	 * @return TableColumnInfo
	 */
	@SuppressWarnings("unchecked")
	public ArrayList getPerformanceListTableDefine(Locale local) {
		
		/** ơ֥ */
		SelectMonitorPerformance selectPerf = new SelectMonitorPerformance();
		ArrayList tableDefine = selectPerf.getMonitorListTableDefine(local);

		return tableDefine;
	}
}
