/*
 
Copyright (C) 2006 NTT DATA Corporation
 
This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, version 2.
 
This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.
 
*/

package com.clustercontrol.snmp.factory;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.jms.JMSException;
import javax.naming.NamingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.bean.PriorityConstant;
import com.clustercontrol.monitor.run.factory.RunMonitorStringValueType;
import com.clustercontrol.repository.bean.FacilityAttributeConstant;
import com.clustercontrol.snmp.ejb.entity.MonitorSnmpInfoLocal;
import com.clustercontrol.snmp.ejb.entity.MonitorSnmpInfoPK;
import com.clustercontrol.snmp.ejb.entity.MonitorSnmpInfoUtil;
import com.clustercontrol.snmp.util.RequestSnmp;
import com.clustercontrol.util.Messages;

/**
 * SNMPƻ ʸƻѼ¹ԥ饹
 *
 * @version 2.1.0
 * @since 2.1.0
 */
public class RunMonitorSnmpString extends RunMonitorStringValueType {
	
	protected static Log m_log = LogFactory.getLog( RunMonitorSnmpString.class );
	
	public static final String MESSAGE_ID_INFO = "001";
	public static final String MESSAGE_ID_WARNING = "002";
	public static final String MESSAGE_ID_CRITICAL = "003";
	public static final String MESSAGE_ID_UNKNOWN = "004";
	
	/** SNMPƻ */
	protected static final ArrayList<String> m_attributeList = new ArrayList<String>();
	static{
		m_attributeList.add(FacilityAttributeConstant.IPNETWORKNUMBER);
		m_attributeList.add(FacilityAttributeConstant.IPNETWORKNUMBERV6);
	}
	
	/** SNMPƻ */
	protected MonitorSnmpInfoLocal m_snmp = null;
	
	/** SNMP */
	protected RequestSnmp m_request = null;
	
	/** ߥ˥ƥ̾ */
	protected String m_communityName = null;
	
	/** OID */
	protected String m_snmpOid = null;
	
	/** ꥸʥå */
	protected String m_messageOrg = null;
	
	/** åID */
	protected String m_messageId = "";
	
	/** å */
	protected String m_message = "";
	
	/**
	 * 󥹥ȥ饯
	 * 
	 */
	public RunMonitorSnmpString() throws NamingException, JMSException, CreateException{
		super();
	}
	
	/**
	 * OIDͤ
	 * 
	 * @param facilityId եƥID
	 * @return ͼ硢true
	 */
	@Override
	public boolean collect(String facilityId) {
		
		m_nodeDate = m_now.getTime();
		m_value = null;
		
		// å
		m_message = "";
		m_messageOrg = Messages.getString("oid") + " : " + m_snmpOid;
		
		try{
			
			// IPɥ쥹μ
			String ipNetworkNumber = null;
			try {
				// Ρɤ°
				HashMap facilityAttrMap = m_repository.getNodeDetail(facilityId, m_attributeList);
				ipNetworkNumber = (String)facilityAttrMap.get(FacilityAttributeConstant.IPNETWORKNUMBER);
			}
			catch(FinderException e){
				m_log.debug("run():" + e.getMessage());
	            
	            m_message = Messages.getString("message.snmp.6");
	            m_messageOrg = m_messageOrg + " (" + e.getMessage() + ")";
				return false;
			}
			catch(NamingException e){
				m_log.debug("run():" + e.getMessage());
	            
	            m_message = Messages.getString("message.snmp.6");
	            m_messageOrg = m_messageOrg + " (" + e.getMessage() + ")";
				return false;
			}
			
			// SNMPͼ
			RequestSnmp m_request = new RequestSnmp();
			boolean result = m_request.polling(InetAddress.getByName(ipNetworkNumber), m_communityName, m_snmpOid);
			
			if(result){
				
				m_value = m_request.getValue();
				m_nodeDate = m_request.getDate();

				m_messageOrg = m_messageOrg + ", " + Messages.getString("select.value") + " : " + m_value;
			}
			else{
				m_message = m_request.getMessage();
			}
			return result;
		}
		catch (UnknownHostException e) {
			m_log.debug("run():" + e.getMessage());

			m_message = Messages.getString("message.snmp.5");
			m_messageOrg = m_messageOrg + " (" + e.getMessage() + ")";
			return false;
		}
	}
	
	/* ( Javadoc)
	 * SNMPƻ
	 * @see com.clustercontrol.monitor.run.factory.OperationNumericValueInfo#setMonitorAdditionInfo()
	 */
	@Override
	public void setCheckInfo() throws FinderException, NamingException{
		
		// SNMPƻ
		MonitorSnmpInfoPK pk = new MonitorSnmpInfoPK(m_monitorId, m_monitorTypeId);
		m_snmp = MonitorSnmpInfoUtil.getLocalHome().findByPrimaryKey(pk);
		
		// SNMPƻ
		m_communityName = m_snmp.getCommunityName().trim();
		m_snmpOid = m_snmp.getSnmpOid().trim();
	}

	/* ( Javadoc)
	 * ΡѥåID
	 * @see com.clustercontrol.monitor.run.factory.OperationMonitor#getMessageId()
	 */
	@Override
	public String getMessageId(int id) {
		
		String messageId = super.getMessageId(id);
		if(messageId == null || "".equals(messageId)){
			return MESSAGE_ID_UNKNOWN;
		}
		return messageId;
	}
	
	/* ( Javadoc)
	 * Ρѥå
	 * @see com.clustercontrol.monitor.run.factory.OperationMonitor#getMessage(int)
	 */
	@Override
	public String getMessage(int id) {
		
		String message = super.getMessage(id);
		if(message == null || "".equals(message)){
			return m_message;
		}
		return message;
	}

	/* ( Javadoc)
	 * Ρѥꥸʥå
	 * @see com.clustercontrol.monitor.run.factory.OperationMonitor#getMessageOrg(int)
	 */
	@Override
	public String getMessageOrg(int id) {
		return m_messageOrg;
	}
	
	/* ( Javadoc)
	 * ѥåID
	 * @see com.clustercontrol.monitor.run.factory.RunMonitor#getMessageIdForScope(int)
	 */
	@Override
	public String getMessageIdForScope(int priority) {
		
		if(priority == PriorityConstant.TYPE_INFO){
			return MESSAGE_ID_INFO;
		}
		else if(priority == PriorityConstant.TYPE_WARNING){
			return MESSAGE_ID_WARNING;
		}
		else if(priority == PriorityConstant.TYPE_CRITICAL){
			return MESSAGE_ID_CRITICAL;
		}
		else{
			return MESSAGE_ID_UNKNOWN;
		}
	}
}
